#
# Copyright (c) 2008 Jens Keiner
# Copyright (c) 2010 Michael Pippig
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
#  If we find the library, set the shell variable `ax_have_fftw3' to `yes'.
#  Otherwise, set `ax_have_fftw3' to `no'.

AC_DEFUN([AX_LIB_FFTW3],
[
  AC_REQUIRE([AX_OPENMP]) 
 
  AC_ARG_WITH(fftw3, [AC_HELP_STRING([--with-fftw3=DIR],
    [compile with fftw3 in DIR])], with_fftw3=$withval, with_fftw3="yes")

  AC_ARG_WITH(fftw3-libdir, [AC_HELP_STRING([--with-fftw3-libdir=DIR],
    [compile with fftw3 library directory DIR])], fftw3_lib_dir=$withval,
    fftw3_lib_dir="yes")

  AC_ARG_WITH(fftw3-includedir, [AC_HELP_STRING([--with-fftw3-includedir=DIR],
    [compile with fftw3 include directory DIR])], fftw3_include_dir=$withval,
    fftw3_include_dir="yes")

  if test "x$with_fftw3" != "xyes"; then
    if test "x${fftw3_include_dir}" = "xyes"; then
      fftw3_include_dir="$with_fftw3/include"
    fi
    if test "x${fftw3_lib_dir}" = "xyes"; then 
      fftw3_lib_dir="$with_fftw3/lib"
    fi
  fi

  if test "x${fftw3_include_dir}" != "xyes"; then
    AX_CHECK_DIR([${fftw3_include_dir}],[],
      [AC_MSG_ERROR([The directory ${fftw3_include_dir} does not exist.])])
    fftw3_CPPFLAGS="-I$fftw3_include_dir"
  else
    fftw3_CPPFLAGS=""
  fi

  if test "x${fftw3_lib_dir}" != "xyes"; then 
    AX_CHECK_DIR([${fftw3_lib_dir}],[],
      [AC_MSG_ERROR([The directory ${fftw3_lib_dir} does not exist.])])
    fftw3_LDFLAGS="-L$fftw3_lib_dir"
  else
    fftw3_LDFLAGS=""
  fi

  saved_CFLAGS="$CFLAGS"
  saved_CPPFLAGS="$CPPFLAGS"
  saved_LDFLAGS="$LDFLAGS"
  CFLAGS="$CFLAGS $OPENMP_CFLAGS"
  CPPFLAGS="$CPPFLAGS $fftw3_CPPFLAGS"
  LDFLAGS="$LDFLAGS $fftw3_LDFLAGS"

  # Check if header is present and usable.
  ax_lib_fftw3=yes
  ax_lib_fftw3_threads=yes
  AC_CHECK_HEADER([fftw3.h], [], [ax_lib_fftw3=no;ax_lib_fftw3_threads=no])

  ax_lib_fftw3_mpi=yes
  AC_CHECK_HEADER([fftw3-mpi.h], [], [ax_lib_fftw3_mpi=no])

  fftw3_LIBS=
  fftw3_threads_LIBS=
  fftw3_mpi_LIBS=

  if test "x$ax_lib_fftw3" = "xyes"; then
    saved_LIBS="$LIBS"

    AC_CHECK_LIB([fftw3], [fftw_execute], [], [ax_lib_fftw3=no])
    test "x${ax_lib_fftw3}" != xno && fftw3_LIBS="-lfftw3"
    AC_CHECK_LIB([fftw3_threads], [fftw_init_threads], [], [ax_lib_fftw3_threads=no])
    test "x${ax_lib_fftw3_threads}" != xno && fftw3_threads_LIBS="-lfftw3_threads"
    AC_CHECK_LIB([fftw3_mpi], [fftw_mpi_init], [], [ax_lib_fftw3_mpi=no])
    test "x${ax_lib_fftw3_mpi}" != xno && fftw3_mpi_LIBS="-lfftw3_mpi"
    
    LIBS="$saved_LIBS"
  fi

  # Restore saved flags.
  CFLAGS="$saved_CFLAGS"
  CPPFLAGS="$saved_CPPFLAGS"
  LDFLAGS="$saved_LDFLAGS"

  AC_SUBST(fftw3_CPPFLAGS)
  AC_SUBST(fftw3_LIBS)
  AC_SUBST(fftw3_threads_LIBS)
  AC_SUBST(fftw3_mpi_LIBS)
  AC_SUBST(fftw3_LDFLAGS)
])
